require "import"
import "android.speech.*"
import "android.content.*"
import "android.widget.*"
import "android.view.*"
import "cjson"
import "android.net.Uri"
import "com.androlua.Ticker"

context = this or service
local prefs = context.getSharedPreferences("language_prefs", Context.MODE_PRIVATE)
local editor = prefs.edit()
local dlg
local languages = {
    "English", "Arabic", "Bengali", "Hindi", "Indonesian", "Kannada",
    "Marathi", "Nepali", "Persian", "Portuguese", "Russian", "Spanish",
    "Tamil", "Telugu", "Urdu", "Thai", "Turkish", "Uzbek", "Balochi", "Vietnamese"
}
local languageCodes = {
    ["English"] = "en",
    ["Arabic"] = "ar",
    ["Bengali"] = "bn",
    ["Hindi"] = "hi",
    ["Indonesian"] = "id",
    ["Kannada"] = "kn",
    ["Marathi"] = "mr",
    ["Nepali"] = "ne",
    ["Persian"] = "fa",
    ["Portuguese"] = "pt",
    ["Russian"] = "ru",
    ["Spanish"] = "es",
    ["Tamil"] = "ta",
    ["Telugu"] = "te",
    ["Urdu"] = "ur",
    ["Thai"] = "th",
    ["Turkish"] = "tr",
    ["Uzbek"] = "uz",
    ["Balochi"] = "bal",
    ["Vietnamese"] = "vi"
}
local setLanguagePhrase = "set language"
local selectedLanguage = prefs.getString("selected_language", "English")
local selectedTargetLanguage = prefs.getString("selected_target_language", "English")

local useAIMode = prefs.getBoolean("use_ai_mode", true)
local addEmoji = prefs.getBoolean("add_emoji", true)
local playSound = prefs.getBoolean("play_sound", true)
local translateText = prefs.getBoolean("translate_text", false)

function trimText(text)
    text = text:gsub("%s*([%.%?%,!])", "%1")
    text = text:gsub("([%p%z])%s+", "%1")
    text = text:gsub("%s+$", "")
    return text
end

function urlEncode(text)
    text = tostring(text)
    text = text:gsub("\n", "\r\n")
    text = text:gsub("([^%w %-%_%.%~])", function(c)
        return string.format("%%%02X", string.byte(c))
    end)
    text = text:gsub(" ", "+")
    return text
end

function handleCommand(text)
    local command = text:lower()

    text = text:gsub("^%l", string.upper)

    if command == setLanguagePhrase then
        showLanguageSelectionDialog()
        return
    end

    local encodedQuestion
    if useAIMode then
        if translateText then
            encodedQuestion = urlEncode("Your task is to processAnd translate this text and return it with proper punctuation.\nmake grammar correction.\n" .. (addEmoji and "add matching emojis.\n" or "") .. "PleaseMake sure to Convert this textFully in  to " .. selectedTargetLanguage .. ".\nand please provide me processed text without your Any descriptionAnything except my translated text. nHere is the text:\n" .. text)
        else
            encodedQuestion = urlEncode("Your task is to process this text and return it with proper punctuation.\nmake grammar correction.\n" .. (addEmoji and "add matching emojis.\n" or "") .. "and please provide me processed text without your confirming description.\nHere is the text:\n" .. text)
        end
        
        local url = "https://darkness.ashlynn.workers.dev/chat/?model=gpt-4o-mini&prompt=" .. encodedQuestion
        Http.get(url, function(status, data)
            if status == 200 then
                local response = cjson.decode(data)
                local cleanedData = response.response or nil
                if cleanedData and cleanedData ~= "" then
                    cleanedData = trimText(cleanedData)
                    local node = service.getEditText()
                    service.insert(node, cleanedData)
                    service.speak(cleanedData)
                else
                    service.speak("No answer available.")
                end
            else
                service.speak("Error! Please try again.")
            end
        end)
    else
        local node = service.getEditText()
        service.insert(node, text)
    end
end

function startSpeechRecognition()
    local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
    if vibrator then
        vibrator.vibrate(80)
    end
    
    if playSound then
        service.playSoundTick()
    end
    
    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
    local recognitionListener = RecognitionListener{
        onResults = function(results)
            local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if resultArray ~= nil and resultArray.size() > 0 then
                local recognizedText = resultArray.get(0)
                handleCommand(recognizedText)
            end
            if playSound then
                service.playSoundTick()
            end
            speechRecognizer.destroy()

            if vibrator then
                vibrator.vibrate(80)
            end
        end,
        onError = function(error)
            service.speak("Speech recognition error! Please try again.")
        end
    }
    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, languageCodes[selectedLanguage])
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())

    speechRecognizer.setRecognitionListener(recognitionListener)
    speechRecognizer.startListening(recognizerIntent)
end

function showLanguageSelectionDialog()
    local dlgLayout = {
        LinearLayout,
        orientation = LinearLayout.VERTICAL,
        layout_width = "fill",
        layout_height = "fill",
        {
            Spinner,
            id = "language_spinner",
            layout_width = "fill",
            layout_height = "wrap_content",
        },
        {
            CheckBox,
            id = "use_ai_checkbox",
            text = "Use AI mode while dictation",
            layout_width = "fill",
            layout_height = "wrap_content",
            checked = useAIMode,
        },
        {
            CheckBox,
            id = "add_emoji_checkbox",
            text = "Add Emoji Automatically",
            layout_width = "fill",
            layout_height = "wrap_content",
            checked = addEmoji,
            enabled = useAIMode,
        },
        {
            CheckBox,
            id = "play_sound_checkbox",
            text = "Play sound at the beginning and end of dictation",
            layout_width = "fill",
            layout_height = "wrap_content",
            checked = playSound,
        },
        {
            CheckBox,
            id = "translate_text_checkbox",
            text = "Translate Dictated Text",
            layout_width = "fill",
            layout_height = "wrap_content",
            checked = translateText,
            enabled = useAIMode,
        },
        {
            TextView,
            text = "Target Language for Translation:",
            layout_width = "wrap_content",
            layout_height = "wrap_content",
        },
        {
            Spinner,
            id = "target_language_spinner",
            layout_width = "fill",
            layout_height = "wrap_content",
            enabled = useAIMode and translateText,
        },
        {
            TextView,
            text = "Developer: Ayush Kumar",
            layout_width = "wrap_content",
            layout_height = "wrap_content",
        },
        {
            Button,
            text = "OK",
            layout_width = "fill",
            layout_height = "wrap_content",
            onClick = function()
                local selected = languages[language_spinner.getSelectedItemPosition() + 1]
                local selectedTarget = languages[target_language_spinner.getSelectedItemPosition() + 1]
                editor.putString("selected_language", selected)
                editor.putString("selected_target_language", selectedTarget)
                editor.putInt("execution_count", 0)
                editor.commit()
                selectedLanguage = selected
                selectedTargetLanguage = selectedTarget
                startSpeechRecognition()
                dlg.dismiss()
            end
        },
        {
            Button,
            text = "Join Telegram Channel",
            layout_width = "fill",
            layout_height = "wrap_content",
            onClick = function()
                local telegramUrl = "https://t.me/jieshuoofficial/"
                local intent = Intent(Intent.ACTION_VIEW, Uri.parse(telegramUrl))
                context.startActivity(intent)
                dlg.dismiss()
            end
        }
    }

    dlg = LuaDialog(this)
    dlg.setView(loadlayout(dlgLayout))
    dlg.setTitle("Select Language for Dictation")
    dlg.show()

    language_spinner.adapter = ArrayAdapter(service, android.R.layout.simple_spinner_item, String(languages))
    target_language_spinner.adapter = ArrayAdapter(service, android.R.layout.simple_spinner_item, String(languages))

    local function provideFeedback(message)
        local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
        if vibrator then
            vibrator.vibrate(100)
        end
        service.postSpeak(35, message)
    end

    use_ai_checkbox.setOnCheckedChangeListener {
        onCheckedChanged = function(view, isChecked)
            useAIMode = isChecked
            editor.putBoolean("use_ai_mode", isChecked)
            editor.commit()
            add_emoji_checkbox.setEnabled(isChecked)
            translate_text_checkbox.setEnabled(isChecked)
            target_language_spinner.setEnabled(isChecked and translateText)
            provideFeedback(isChecked and "AI mode has been enabled" or "AI mode has been disabled")
        end
    }

    add_emoji_checkbox.setOnCheckedChangeListener {
        onCheckedChanged = function(view, isChecked)
            addEmoji = isChecked
            editor.putBoolean("add_emoji", isChecked)
            editor.commit()
            provideFeedback(isChecked and "Emoji mode has been enabled" or "Emoji mode has been disabled")
        end
    }

    play_sound_checkbox.setOnCheckedChangeListener {
        onCheckedChanged = function(view, isChecked)
            playSound = isChecked
            editor.putBoolean("play_sound", isChecked)
            editor.commit()
            provideFeedback(isChecked and "Sound feedback has been enabled" or "Sound feedback has been disabled")
        end
    }

    translate_text_checkbox.setOnCheckedChangeListener {
        onCheckedChanged = function(view, isChecked)
            translateText = isChecked
            editor.putBoolean("translate_text", isChecked)
            editor.commit()
            target_language_spinner.setEnabled(useAIMode and isChecked)
            provideFeedback(isChecked and "Text translation has been enabled" or "Text translation has been disabled")
        end
    }

    for i, lang in ipairs(languages) do
        if lang == selectedLanguage then
            language_spinner.setSelection(i - 1)
        end
        if lang == selectedTargetLanguage then
            target_language_spinner.setSelection(i - 1)
        end
    end
end

local connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE)
local activeNetworkInfo = connectivityManager.getActiveNetworkInfo()

if activeNetworkInfo == nil or not activeNetworkInfo.isConnected() then
    service.speak("Internet connection is required!")
else
    if service.getEditText() then
        if ti then
            ti.stop()
            showLanguageSelectionDialog()
            ti = nil
            return true
        end
        ti = Ticker()
        ti.Period = 10000
        ti.onTick = function()
            ti.stop()
            ti = nil
        end
        ti.start()
        startSpeechRecognition()
    else
        service.speak("Text box not available on the screen.")
    end
end